names_of_stations = {0:"St-Denis",1:"Hotel-de-ville",2:"St-Laurent",
                     3:"St-Urbain",4:"Parc",5:"University"}

bus_capacity=5       # maximum number of passengers on bus
start_frequency = 2  # one departure every 2 minutes

# The data about who's waiting at each stop, and where they want to go
# dictionary with key=stationID (integer)
# value = list of destinations of people waiting
waiting_at_stop = {   
    0: [1,3,4,3,5,5,5,4,2,5,5],
    1: [2,4,4,3,4,4,2,4,4,3,4,4,2],
    2: [4,5,5,3],
    3: [5,5,4,5,5,5,4,5,4],
    4: [5,5,5,5,5,5],
    5: []
}

# variables to keep track of the simulation
simulation_duration = 20  # number of minutes to be simulated
bus_content = {}  # dictionary with key=busIDs, 
                  # value = list of destinations of passengers on bus
bus_positions = {} # dictionary with key=busIDs, 
                   # value = stationID

def print_waiting_at_stop(waiting_at_stop):    
    print("\nCustomers waiting at stop:"); 
    for k,x in waiting_at_stop.items():
        print(f"{k}: {x}")
        
def print_bus_positions(bus_positions):
    print("\nBus positions:"); 
    for k,x in bus_positions.items():
        print(f"{k}: {x}")
        
def print_bus_content(bus_content):
    print("\nPassengers on the bus:"); 
    for k,x in bus_content.items():
        print(f"{k}: {x}")    

# printing the state of the simulation, for debugging purposes
def print_current_state(bus_positions, bus_content, waiting_at_stop):
    print_bus_positions(bus_positions)
    print_bus_content(bus_content)
    print_waiting_at_stop(waiting_at_stop)


print(f"\n****Initial State****\n")
print_current_state(bus_positions, bus_content, waiting_at_stop)
#input("Continue [hit Enter]")

# simulation loop
for time in range(0,simulation_duration): 
    
    print(f"\n****Time {time}****")
   
    # move the buses up by one station
    for bus,pos in bus_positions.items():
        if pos < len(names_of_stations)-1:
            bus_positions[bus]+=1
            
    
    # bring new bus to first station at every start_frequency minutes
    if time % start_frequency == 0 :  # time is a multiple of start_frequency
        bus_id = "bus_that_started_at_time"+str(time)
        bus_positions[bus_id] = 0  # starts at station 0
        bus_content[bus_id] = []  # empty bus      
    
        
    # let people disembark if they are at their station
    for bus_id,content in bus_content.items():
        station = bus_positions[bus_id]  # this is where the bus is
        bus_content[bus_id] = [dest for dest in bus_content[bus_id] if dest!=station] 
    
       
    # let people embark, until the bus is full
    for bus_id,content in bus_content.items():
        station = bus_positions[bus_id]
        number_boarding = min(len(waiting_at_stop[station]), bus_capacity-len(content))
        people_boarding = waiting_at_stop[station][0:number_boarding]
        bus_content[bus_id].extend(people_boarding) # extend the existing passengers
        del waiting_at_stop[station][0:number_boarding]
        
        
    print_current_state(bus_positions, bus_content, waiting_at_stop)
    
    