import matplotlib.pyplot as plt

names_of_stations = {0:"St-Denis",1:"Hotel-de-ville",2:"St-Laurent",
                     3:"St-Urbain",4:"Parc",5:"University"}

bus_capacity=5       # maximum number of passengers on bus
start_frequency = 2  # one departure every 2 minutes

# The data about who's waiting at each stop, and where they want to go
# dictionary with key=stationID (integer)
# value = list of destinations of people waiting
waiting_at_stop = {   
    0: [1,3,4,3,5,5,5,4,2,5,5],
    1: [2,4,4,3,4,4,2,4,4,3,4,4,2],
    2: [4,5,5,3],
    3: [5,5,4,5,5,5,4,5,4],
    4: [5,5,5,5,5,5],
    5: []
}

# variables to keep track of the simulation
simulation_duration = 20  # number of minutes to be simulated
bus_content = {}  # dictionary with key=busIDs, 
                  # value = list of destinations of passengers on bus
bus_positions = {} # dictionary with key=busIDs, 
                   # value = stationID

# For our statistical analyses
nb_arrivals_over_time = {} # dictionary, with key = stationID (integer)
                           # value = number of people who arrived at the station
nb_waiting_over_time = {}  #dictionary, with key = stationID (integer)
                           # value = list of number of people waiting at station
                           #         per time point

# initializing our counters to zero
for station in range(0,len(names_of_stations)):
    nb_arrivals_over_time[station]=[0]*simulation_duration
    nb_waiting_over_time[station]=[0]*simulation_duration


# simulation loop
for time in range(0,simulation_duration): 
    
    # record how many people are still waiting?
    for station, waiting in waiting_at_stop.items():
        nb_waiting_over_time[station][time]=len(waiting)
       
    # move the buses up by one station
    for bus,pos in bus_positions.items():
        if pos< len(names_of_stations)-1:
            bus_positions[bus]+=1
            
    # bring new bus to first station at every start_frequency minutes
    if time % start_frequency == 0 :  # time is a multiple of start_frequency
        bus_id = "bus_that_started_at_time"+str(time)
        bus_positions[bus_id] = 0  # starts at station 0
        bus_content[bus_id] = []  # empty bus      
    
    #print("Before disembarking")
    
    # let people disembark if they are at their station
    for bus_id,content in bus_content.items():
        station = bus_positions[bus_id]  # this is where the bus is
        
        # record how many people are arriving at the current station
        nb_disembark = bus_content[bus_id].count(station) # count how many dest == station
        nb_arrivals_over_time[station][time]=nb_disembark # save it at the station and time
        
        bus_content[bus_id] = [dest for dest in bus_content[bus_id] if dest!=station] 
    
    #print("Before loading")
    
    # let people embark, until the bus is full
    for bus_id,content in bus_content.items():
        station = bus_positions[bus_id]
        number_boarding = min(len(waiting_at_stop[station]),bus_capacity-len(content))
        people_boarding = waiting_at_stop[station][0:number_boarding]
        bus_content[bus_id].extend(people_boarding)
        del waiting_at_stop[station][0:number_boarding]
    

# plot the number of people waiting at each station, as a function of time
for stationID, stationName in names_of_stations.items():
    plt.plot(nb_waiting_over_time[stationID], label=stationName)
    
plt.xlabel('Time')
plt.ylabel('Number of people waiting')
plt.xticks(range(0,simulation_duration))
plt.legend()
plt.savefig("waiting.eps")
#plt.show()


plt.clf()  # clears figure to generate a new one
plt.rcParams['figure.figsize'] = (16,9)
for stationID, stationName in names_of_stations.items():
    plt.subplot(2,3, stationID+1)
    plt.bar(range(len(nb_arrivals_over_time[stationID])), 
            nb_arrivals_over_time[stationID], label=stationName)
    plt.title(stationName)
    plt.xlabel('Time')
    plt.ylabel("Number of people arriving")
    plt.xticks(range(0,simulation_duration))
    plt.ylim(bottom=0, top=5)

plt.savefig("arrivals.eps")
#plt.show()



















